import argparse
import sys
import os
import random

import dcs


def main():
    red = [
        (dcs.countries.Germany.name, dcs.countries.Germany.Plane.Bf_109K_4.id),
        (dcs.countries.Germany.name, dcs.countries.Germany.Plane.FW_190D9.id),
    ]

    blue = [
        (dcs.countries.USA.name, dcs.countries.USA.Plane.P_51D.id)
    ]

    types = red + blue

    aircraft_types = [x[1] for x in types]

    parser = argparse.ArgumentParser(description="DCS WWII dogfight generator")
    parser.add_argument("-a", "--aircrafttype", default=dcs.planes.Bf_109_K_4.id,
                        choices=aircraft_types,
                        help="Player aircraft type")
    parser.add_argument("-p", "--playercount", default=1, type=int)
    parser.add_argument("-n", "--numberplanes", default=16, type=int, help="Count of planes per side")
    parser.add_argument("-t", "--terrain", choices=["caucasus", "nevada"], default='caucasus')
    parser.add_argument("-s", "--skill", choices=[x.value for x in dcs.unit.Skill], default=dcs.unit.Skill.Average.value)
    parser.add_argument("-o", "--output", help="Name and path of the generated mission", default=None)

    args = parser.parse_args()
    terrain_map = {
        "caucasus": dcs.terrain.Caucasus,
        "nevada": dcs.terrain.Nevada
    }

    if args.output is None:
        if args.terrain == "caucasus":
            args.output = os.path.join(os.path.expanduser("~"), "Saved Games\\DCS\\Missions\\dogfight_wwii.miz")
        else:
            args.output = os.path.join(os.path.expanduser("~"), "Saved Games\\DCS.openalpha\\Missions\\dogfight_wwii.miz")

    m = dcs.Mission(terrain_map[args.terrain]())

    m.coalition["blue"].swap_country(m.coalition["red"], dcs.countries.Germany.name)

    # pick a random airport as fight reference
    airport = random.choice(list(m.terrain.airport_list()))
    battle_point = dcs.Rectangle.from_point(airport.position, 40 * 1000).random_point()
    heading = random.randrange(0, 360)

    # set editor mapview
    m.map.position = battle_point
    m.map.zoom = 100000

    # fight altitude
    altitude = random.randrange(2000, 6000, 100)

    if args.playercount == 1:
        player_country = m.country([x[0] for x in types if x[1] == args.aircrafttype][0])
        hdg = heading + 180 if m.is_blue(player_country) else heading
        fg = m.flight_group_inflight(player_country, "Player " + args.aircrafttype,
                                     dcs.planes.plane_map[args.aircrafttype],
                                     position=battle_point.point_from_heading(hdg, 10*800),
                                     altitude=altitude, speed=500, maintask=dcs.task.Intercept)
        fg.add_waypoint(battle_point, altitude=altitude)
        fg.units[0].set_player()
    else:
        for x in types:
            country = m.country(x[0])
            hdg = heading + 180 if m.is_blue(country) else heading
            pos = battle_point.point_from_heading(hdg, 10 * 800)
            pos = pos.point_from_heading(hdg + 90, random.randrange(-5000, 5000, 100))
            fg = m.flight_group_inflight(country, x[1] + " group",
                                         dcs.planes.plane_map[x[1]],
                                         position=pos, altitude=random.randrange(altitude - 200, altitude + 200),
                                         speed=500,
                                         maintask=dcs.task.FighterSweep,
                                         group_size=args.playercount)
            fg.set_client()
            fg.add_waypoint(battle_point, altitude=altitude)

    # spawn AI
    group_count = int(args.numberplanes / 4)
    last_group_size = args.numberplanes % 4

    for c in [red, blue]:
        gc = 1
        for x in range(0, group_count):
            planetype = random.choice(c)
            country = m.country(planetype[0])
            hdg = heading + 180 if m.is_blue(country) else heading
            pos = battle_point.point_from_heading(hdg, 10*800)
            pos = pos.point_from_heading(hdg+90, random.randrange(-5000, 5000, 100))
            fg = m.flight_group_inflight(country, planetype[0] + " " + planetype[1] + " #" + str(gc),
                                         dcs.planes.plane_map[planetype[1]],
                                         position=pos, altitude=random.randrange(altitude - 400, altitude + 400, 50),
                                         speed=500,
                                         maintask=dcs.task.FighterSweep,
                                         group_size=4)
            fg.set_skill(dcs.unit.Skill(args.skill))
            fg.add_waypoint(battle_point, altitude=altitude)
            gc += 1

        if last_group_size:
            planetype = random.choice(c)
            country = m.country(planetype[0])
            hdg = heading + 180 if m.is_blue(country) else heading
            pos = battle_point.point_from_heading(hdg, 10 * 800)
            pos = pos.point_from_heading(hdg + 90, random.randrange(-5000, 5000, 100))
            fg = m.flight_group_inflight(m.country(planetype[0]), planetype[0] + " " + planetype[1] + " #" + str(gc),
                                         dcs.planes.plane_map[planetype[1]],
                                         position=pos, altitude=random.randrange(altitude - 400, altitude + 400, 50),
                                         speed=500,
                                         maintask=dcs.task.FighterSweep,
                                         group_size=last_group_size)
            fg.set_skill(dcs.unit.Skill(args.skill))
            fg.add_waypoint(battle_point, altitude=altitude)

    m.set_sortie_text("WWII dogfight")
    stats = m.stats()
    m.set_description_text("""A WWII dogfight encounter
There are {pc} planes in the air battling for life and death.""".format(
        pc=stats["red"]["plane_groups"]["unit_count"]+stats["blue"]["plane_groups"]["unit_count"]))
    m.set_description_redtask_text("Fight the other planes!")
    m.set_description_bluetask_text("Fight the other planes!")

    m.save(args.output)

    print("Mission created: " + args.output)
    return 0

if __name__ == '__main__':
    sys.exit(main())



from . import lua
import os
import re


class UnitType:
    id = None
    name = None


class VehicleType(UnitType):
    eplrs = False


class ShipType(UnitType):
    helicopter_num = 0
    plane_num = 0
    parking = 0


class StaticType(UnitType):
    shape_name = None
    rate = 0
    category = "Fortifications"
    sea_object = False
    can_cargo = False
    mass = None


class FlyingType(UnitType):
    flyable = False
    group_size_max = 4
    large_parking_slot = False
    helicopter = False
    fuel_max = 0
    max_speed = 500
    height = 0
    width = 0
    length = 0
    ammo_type = None
    chaff = 0
    flare = 0
    charge_total = 0
    chaff_charge_size = 1
    flare_charge_size = 2
    category = "Air"

    tacan = False
    eplrs = False

    radio_frequency = 251
    panel_radio = None

    property_defaults = None

    pylons = {}
    payloads = None
    dcs_dir = "C:\\Program Files\\Eagle Dynamics\\DCS World\\"
    payload_dirs = [
        dcs_dir + "MissionEditor\\data\\scripts\\UnitPayloads",
        dcs_dir + "CoreMods\\aircraft\\M-2000C\\UnitPayloads",
        dcs_dir + "CoreMods\\aircraft\\MiG-21BIS\\UnitPayloads",
        dcs_dir + "CoreMods\\aircraft\\F-5E\\UnitPayloads",
        dcs_dir + "CoreMods\\aircraft\\C-101\\UnitPayloads",
        os.path.join(os.path.expanduser("~"), "Saved Games\\DCS\\MissionEditor\\UnitPayloads"),
        os.path.join(os.path.dirname(os.path.realpath(__file__)), "payloads")
    ]

    tasks = ['Nothing']
    task_default = None

    _payload_cache = None

    @classmethod
    def scan_payload_dir(cls):
        if FlyingType._payload_cache:
            return
        FlyingType._payload_cache = {}
        for payload_dir in FlyingType.payload_dirs:
            if not os.path.exists(payload_dir):
                continue
            files = [file for file in os.listdir(payload_dir) if file.endswith('.lua')]
            for file in files:
                payload_filename = os.path.join(payload_dir, file)
                if payload_filename not in FlyingType._payload_cache:
                    with open(payload_filename, 'r') as payloadfile:
                        for line in payloadfile:
                            g = re.search(r'\["unitType"\]\s*=\s*"([^"]*)', line)
                            if g:
                                FlyingType._payload_cache[payload_filename] = g.group(1)
                                break

    @classmethod
    def load_payloads(cls):
        FlyingType.scan_payload_dir()
        if cls.payloads:
            return cls.payloads

        for payload_dir in FlyingType.payload_dirs:
            if not os.path.exists(payload_dir):
                continue
            files = [file for file in os.listdir(payload_dir) if file.endswith('.lua')]
            for file in files:
                payload_filename = os.path.join(payload_dir, file)
                if FlyingType._payload_cache[payload_filename] == cls.id and os.path.exists(payload_filename):
                    with open(payload_filename, 'r') as payload:
                        payload_main = lua.loads(payload.read())
                        pays = payload_main[list(payload_main.keys())[0]]
                        if pays["unitType"] == cls.id:
                            if cls.payloads:
                                highestkey = max(pays["payloads"].keys()) + 1
                                for load in pays["payloads"]:
                                    x = [x for x in cls.payloads["payloads"]
                                         if cls.payloads["payloads"][x]["name"] == pays["payloads"][load]["name"]]
                                    if x:
                                        cls.payloads["payloads"][x[0]] = pays["payloads"][load]
                                    else:
                                        cls.payloads["payloads"][highestkey] = pays["payloads"][load]
                                        highestkey += 1
                            else:
                                cls.payloads = pays

        return cls.payloads

    @classmethod
    def loadout(cls, _task):
        if cls.payloads:
            for p in cls.payloads["payloads"]:
                payload = cls.payloads["payloads"][p]
                tasks = [payload["tasks"][x] for x in payload["tasks"]]
                if _task.id in tasks:
                    pylons = payload["pylons"]
                    r = [(pylons[x]["num"], {"clsid": pylons[x]["CLSID"]}) for x in pylons]
                    return r
        return None

    @classmethod
    def loadout_by_name(cls, loadout_name):
        if cls.payloads:
            for p in cls.payloads["payloads"]:
                payload = cls.payloads["payloads"][p]
                if payload["name"] == loadout_name:
                    pylons = payload["pylons"]
                    r = [(pylons[x]["num"], {"clsid": pylons[x]["CLSID"]}) for x in pylons]
                    return r
        return None

    @classmethod
    def default_livery(cls, country_name) -> str:
        liveries = cls.Liveries
        for x in liveries.__dict__:
            clas = liveries.__dict__[x]
            if clas and getattr(clas, "__name__", "") == country_name:
                return list(clas)[0].value
        return None



class Condition:
    def __init__(self, predicate):
        self.predicate = predicate
        self.params = []

    def __repr__(self):
        return self.predicate + "(" + ",".join(map(str, self.params)) + ")"

    def dict(self):
        d = {
            "predicate": self.predicate
        }
        return d


class AllOfCoalitionInZone(Condition):
    predicate = "c_all_of_coalition_in_zone"

    def __init__(self, coalitionlist, zone):
        super(AllOfCoalitionInZone, self).__init__(AllOfCoalitionInZone.predicate)
        self.coalitionlist = coalitionlist
        self.params.append(self.coalitionlist)
        self.zone = zone
        self.params.append(self.zone)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["coalitionlist"], d["zone"])

    def dict(self):
        d = super(AllOfCoalitionInZone, self).dict()
        d["coalitionlist"] = self.coalitionlist
        d["zone"] = self.zone
        return d


class AllOfCoalitionOutsideZone(Condition):
    predicate = "c_all_of_coalition_out_zone"

    def __init__(self, coalitionlist, zone):
        super(AllOfCoalitionOutsideZone, self).__init__(AllOfCoalitionOutsideZone.predicate)
        self.coalitionlist = coalitionlist
        self.params.append(self.coalitionlist)
        self.zone = zone
        self.params.append(self.zone)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["coalitionlist"], d["zone"])

    def dict(self):
        d = super(AllOfCoalitionOutsideZone, self).dict()
        d["coalitionlist"] = self.coalitionlist
        d["zone"] = self.zone
        return d


class AllOfGroupInZone(Condition):
    predicate = "c_all_of_group_in_zone"

    def __init__(self, group, zone):
        super(AllOfGroupInZone, self).__init__(AllOfGroupInZone.predicate)
        self.group = group
        self.params.append(self.group)
        self.zone = zone
        self.params.append(self.zone)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["group"], d["zone"])

    def dict(self):
        d = super(AllOfGroupInZone, self).dict()
        d["group"] = self.group
        d["zone"] = self.zone
        return d


class AllOfGroupOutsideZone(Condition):
    predicate = "c_all_of_group_out_zone"

    def __init__(self, group, zone):
        super(AllOfGroupOutsideZone, self).__init__(AllOfGroupOutsideZone.predicate)
        self.group = group
        self.params.append(self.group)
        self.zone = zone
        self.params.append(self.zone)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["group"], d["zone"])

    def dict(self):
        d = super(AllOfGroupOutsideZone, self).dict()
        d["group"] = self.group
        d["zone"] = self.zone
        return d


class ArgumentInRange(Condition):
    predicate = "c_argument_in_range"

    def __init__(self, argument, _min, _max):
        super(ArgumentInRange, self).__init__(ArgumentInRange.predicate)
        self.argument = argument
        self.params.append(self.argument)
        self._min = _min
        self.params.append(self._min)
        self._max = _max
        self.params.append(self._max)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["argument"], d["_min"], d["_max"])

    def dict(self):
        d = super(ArgumentInRange, self).dict()
        d["argument"] = self.argument
        d["_min"] = self._min
        d["_max"] = self._max
        return d


class BombInZone(Condition):
    predicate = "c_bomb_in_zone"

    def __init__(self, typebomb, numbombs, zone):
        super(BombInZone, self).__init__(BombInZone.predicate)
        self.typebomb = typebomb
        self.params.append(self.typebomb)
        self.numbombs = numbombs
        self.params.append(self.numbombs)
        self.zone = zone
        self.params.append(self.zone)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["typebomb"], d["numbombs"], d["zone"])

    def dict(self):
        d = super(BombInZone, self).dict()
        d["typebomb"] = self.typebomb
        d["numbombs"] = self.numbombs
        d["zone"] = self.zone
        return d


class CargoUnhookedInZone(Condition):
    predicate = "c_cargo_unhooked_in_zone"

    def __init__(self, cargo, zone):
        super(CargoUnhookedInZone, self).__init__(CargoUnhookedInZone.predicate)
        self.cargo = cargo
        self.params.append(self.cargo)
        self.zone = zone
        self.params.append(self.zone)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["cargo"], d["zone"])

    def dict(self):
        d = super(CargoUnhookedInZone, self).dict()
        d["cargo"] = self.cargo
        d["zone"] = self.zone
        return d


class CoalitionHasAirdrome(Condition):
    predicate = "c_coalition_has_airdrome"

    def __init__(self, coalitionlist, airdromelist):
        super(CoalitionHasAirdrome, self).__init__(CoalitionHasAirdrome.predicate)
        self.coalitionlist = coalitionlist
        self.params.append(self.coalitionlist)
        self.airdromelist = airdromelist
        self.params.append(self.airdromelist)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["coalitionlist"], d["airdromelist"])

    def dict(self):
        d = super(CoalitionHasAirdrome, self).dict()
        d["coalitionlist"] = self.coalitionlist
        d["airdromelist"] = self.airdromelist
        return d


class CoalitionHasHelipad(Condition):
    predicate = "c_coalition_has_helipad"

    def __init__(self, coalitionlist, helipadlist):
        super(CoalitionHasHelipad, self).__init__(CoalitionHasHelipad.predicate)
        self.coalitionlist = coalitionlist
        self.params.append(self.coalitionlist)
        self.helipadlist = helipadlist
        self.params.append(self.helipadlist)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["coalitionlist"], d["helipadlist"])

    def dict(self):
        d = super(CoalitionHasHelipad, self).dict()
        d["coalitionlist"] = self.coalitionlist
        d["helipadlist"] = self.helipadlist
        return d


class CockpitHighlightVisible(Condition):
    predicate = "c_cockpit_highlight_visible"

    def __init__(self, highlight_id):
        super(CockpitHighlightVisible, self).__init__(CockpitHighlightVisible.predicate)
        self.highlight_id = highlight_id
        self.params.append(self.highlight_id)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["highlight_id"])

    def dict(self):
        d = super(CockpitHighlightVisible, self).dict()
        d["highlight_id"] = self.highlight_id
        return d


class CockpitParamEqual(Condition):
    predicate = "c_cockpit_param_equal_to"

    def __init__(self, cockpit_param, value_text):
        super(CockpitParamEqual, self).__init__(CockpitParamEqual.predicate)
        self.cockpit_param = cockpit_param
        self.params.append(self.cockpit_param)
        self.value_text = value_text
        self.params.append(self.value_text)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["cockpit_param"], d["value_text"])

    def dict(self):
        d = super(CockpitParamEqual, self).dict()
        d["cockpit_param"] = self.cockpit_param
        d["value_text"] = self.value_text
        return d


class CockpitParamEqualToAnother(Condition):
    predicate = "c_cockpit_param_is_equal_to_another"

    def __init__(self, cockpit_param, cockpit_param2):
        super(CockpitParamEqualToAnother, self).__init__(CockpitParamEqualToAnother.predicate)
        self.cockpit_param = cockpit_param
        self.params.append(self.cockpit_param)
        self.cockpit_param2 = cockpit_param2
        self.params.append(self.cockpit_param2)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["cockpit_param"], d["cockpit_param2"])

    def dict(self):
        d = super(CockpitParamEqualToAnother, self).dict()
        d["cockpit_param"] = self.cockpit_param
        d["cockpit_param2"] = self.cockpit_param2
        return d


class CockpitParamInRange(Condition):
    predicate = "c_cockpit_param_in_range"

    def __init__(self, cockpit_param, _min2, _max2):
        super(CockpitParamInRange, self).__init__(CockpitParamInRange.predicate)
        self.cockpit_param = cockpit_param
        self.params.append(self.cockpit_param)
        self._min2 = _min2
        self.params.append(self._min2)
        self._max2 = _max2
        self.params.append(self._max2)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["cockpit_param"], d["_min2"], d["_max2"])

    def dict(self):
        d = super(CockpitParamInRange, self).dict()
        d["cockpit_param"] = self.cockpit_param
        d["_min2"] = self._min2
        d["_max2"] = self._max2
        return d


class FlagEquals(Condition):
    predicate = "c_flag_equals"

    def __init__(self, flag=1, value=10):
        super(FlagEquals, self).__init__(FlagEquals.predicate)
        self.flag = flag
        self.params.append(self.flag)
        self.value = value
        self.params.append(self.value)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["flag"], d["value"])

    def dict(self):
        d = super(FlagEquals, self).dict()
        d["flag"] = self.flag
        d["value"] = self.value
        return d


class FlagEqualsFlag(Condition):
    predicate = "c_flag_equals_flag"

    def __init__(self, flag=1, flag2=1):
        super(FlagEqualsFlag, self).__init__(FlagEqualsFlag.predicate)
        self.flag = flag
        self.params.append(self.flag)
        self.flag2 = flag2
        self.params.append(self.flag2)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["flag"], d["flag2"])

    def dict(self):
        d = super(FlagEqualsFlag, self).dict()
        d["flag"] = self.flag
        d["flag2"] = self.flag2
        return d


class FlagIsFalse(Condition):
    predicate = "c_flag_is_false"

    def __init__(self, flag=1):
        super(FlagIsFalse, self).__init__(FlagIsFalse.predicate)
        self.flag = flag
        self.params.append(self.flag)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["flag"])

    def dict(self):
        d = super(FlagIsFalse, self).dict()
        d["flag"] = self.flag
        return d


class FlagIsLess(Condition):
    predicate = "c_flag_less"

    def __init__(self, flag=1, value=10):
        super(FlagIsLess, self).__init__(FlagIsLess.predicate)
        self.flag = flag
        self.params.append(self.flag)
        self.value = value
        self.params.append(self.value)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["flag"], d["value"])

    def dict(self):
        d = super(FlagIsLess, self).dict()
        d["flag"] = self.flag
        d["value"] = self.value
        return d


class FlagIsLessThanFlag(Condition):
    predicate = "c_flag_less_flag"

    def __init__(self, flag=1, flag2=1):
        super(FlagIsLessThanFlag, self).__init__(FlagIsLessThanFlag.predicate)
        self.flag = flag
        self.params.append(self.flag)
        self.flag2 = flag2
        self.params.append(self.flag2)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["flag"], d["flag2"])

    def dict(self):
        d = super(FlagIsLessThanFlag, self).dict()
        d["flag"] = self.flag
        d["flag2"] = self.flag2
        return d


class FlagIsMore(Condition):
    predicate = "c_flag_more"

    def __init__(self, flag=1, value=10):
        super(FlagIsMore, self).__init__(FlagIsMore.predicate)
        self.flag = flag
        self.params.append(self.flag)
        self.value = value
        self.params.append(self.value)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["flag"], d["value"])

    def dict(self):
        d = super(FlagIsMore, self).dict()
        d["flag"] = self.flag
        d["value"] = self.value
        return d


class FlagIsTrue(Condition):
    predicate = "c_flag_is_true"

    def __init__(self, flag=1):
        super(FlagIsTrue, self).__init__(FlagIsTrue.predicate)
        self.flag = flag
        self.params.append(self.flag)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["flag"])

    def dict(self):
        d = super(FlagIsTrue, self).dict()
        d["flag"] = self.flag
        return d


class GroupAlive(Condition):
    predicate = "c_group_alive"

    def __init__(self, group):
        super(GroupAlive, self).__init__(GroupAlive.predicate)
        self.group = group
        self.params.append(self.group)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["group"])

    def dict(self):
        d = super(GroupAlive, self).dict()
        d["group"] = self.group
        return d


class GroupDead(Condition):
    predicate = "c_group_dead"

    def __init__(self, group):
        super(GroupDead, self).__init__(GroupDead.predicate)
        self.group = group
        self.params.append(self.group)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["group"])

    def dict(self):
        d = super(GroupDead, self).dict()
        d["group"] = self.group
        return d


class GroupLifeLess(Condition):
    predicate = "c_group_life_less"

    def __init__(self, group, percent=10):
        super(GroupLifeLess, self).__init__(GroupLifeLess.predicate)
        self.group = group
        self.params.append(self.group)
        self.percent = percent
        self.params.append(self.percent)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["group"], d["percent"])

    def dict(self):
        d = super(GroupLifeLess, self).dict()
        d["group"] = self.group
        d["percent"] = self.percent
        return d


class IndicationTextEqual(Condition):
    predicate = "c_indication_txt_equal_to"

    def __init__(self, indicator_id, element_name, element_value):
        super(IndicationTextEqual, self).__init__(IndicationTextEqual.predicate)
        self.indicator_id = indicator_id
        self.params.append(self.indicator_id)
        self.element_name = element_name
        self.params.append(self.element_name)
        self.element_value = element_value
        self.params.append(self.element_value)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["indicator_id"], d["element_name"], d["element_value"])

    def dict(self):
        d = super(IndicationTextEqual, self).dict()
        d["indicator_id"] = self.indicator_id
        d["element_name"] = self.element_name
        d["element_value"] = self.element_value
        return d


class MissionScoreHigher(Condition):
    predicate = "c_mission_score_higher"

    def __init__(self, coalitionlist, score=50):
        super(MissionScoreHigher, self).__init__(MissionScoreHigher.predicate)
        self.coalitionlist = coalitionlist
        self.params.append(self.coalitionlist)
        self.score = score
        self.params.append(self.score)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["coalitionlist"], d["score"])

    def dict(self):
        d = super(MissionScoreHigher, self).dict()
        d["coalitionlist"] = self.coalitionlist
        d["score"] = self.score
        return d


class MissionScoreLower(Condition):
    predicate = "c_mission_score_lower"

    def __init__(self, coalitionlist, score=50):
        super(MissionScoreLower, self).__init__(MissionScoreLower.predicate)
        self.coalitionlist = coalitionlist
        self.params.append(self.coalitionlist)
        self.score = score
        self.params.append(self.score)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["coalitionlist"], d["score"])

    def dict(self):
        d = super(MissionScoreLower, self).dict()
        d["coalitionlist"] = self.coalitionlist
        d["score"] = self.score
        return d


class Or(Condition):
    predicate = "or"

    def __init__(self, ):
        super(Or, self).__init__(Or.predicate)


    @classmethod
    def create_from_dict(cls, d):
        return cls()

    def dict(self):
        d = super(Or, self).dict()

        return d


class PartOfCoalitionInZone(Condition):
    predicate = "c_part_of_coalition_in_zone"

    def __init__(self, coalitionlist, zone):
        super(PartOfCoalitionInZone, self).__init__(PartOfCoalitionInZone.predicate)
        self.coalitionlist = coalitionlist
        self.params.append(self.coalitionlist)
        self.zone = zone
        self.params.append(self.zone)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["coalitionlist"], d["zone"])

    def dict(self):
        d = super(PartOfCoalitionInZone, self).dict()
        d["coalitionlist"] = self.coalitionlist
        d["zone"] = self.zone
        return d


class PartOfCoalitionOutsideZone(Condition):
    predicate = "c_part_of_coalition_out_zone"

    def __init__(self, coalitionlist, zone):
        super(PartOfCoalitionOutsideZone, self).__init__(PartOfCoalitionOutsideZone.predicate)
        self.coalitionlist = coalitionlist
        self.params.append(self.coalitionlist)
        self.zone = zone
        self.params.append(self.zone)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["coalitionlist"], d["zone"])

    def dict(self):
        d = super(PartOfCoalitionOutsideZone, self).dict()
        d["coalitionlist"] = self.coalitionlist
        d["zone"] = self.zone
        return d


class PartOfGroupInZone(Condition):
    predicate = "c_part_of_group_in_zone"

    def __init__(self, group, zone):
        super(PartOfGroupInZone, self).__init__(PartOfGroupInZone.predicate)
        self.group = group
        self.params.append(self.group)
        self.zone = zone
        self.params.append(self.zone)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["group"], d["zone"])

    def dict(self):
        d = super(PartOfGroupInZone, self).dict()
        d["group"] = self.group
        d["zone"] = self.zone
        return d


class PartOfGroupOutsideZone(Condition):
    predicate = "c_part_of_group_out_zone"

    def __init__(self, group, zone):
        super(PartOfGroupOutsideZone, self).__init__(PartOfGroupOutsideZone.predicate)
        self.group = group
        self.params.append(self.group)
        self.zone = zone
        self.params.append(self.zone)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["group"], d["zone"])

    def dict(self):
        d = super(PartOfGroupOutsideZone, self).dict()
        d["group"] = self.group
        d["zone"] = self.zone
        return d


class PlayerScoreLess(Condition):
    predicate = "c_player_score_less"

    def __init__(self, scores=100):
        super(PlayerScoreLess, self).__init__(PlayerScoreLess.predicate)
        self.scores = scores
        self.params.append(self.scores)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["scores"])

    def dict(self):
        d = super(PlayerScoreLess, self).dict()
        d["scores"] = self.scores
        return d


class PlayerScoreMore(Condition):
    predicate = "c_player_score_more"

    def __init__(self, scores=100):
        super(PlayerScoreMore, self).__init__(PlayerScoreMore.predicate)
        self.scores = scores
        self.params.append(self.scores)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["scores"])

    def dict(self):
        d = super(PlayerScoreMore, self).dict()
        d["scores"] = self.scores
        return d


class Predicate(Condition):
    predicate = "c_predicate"

    def __init__(self, text):
        super(Predicate, self).__init__(Predicate.predicate)
        self.text = text
        self.params.append(self.text)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["text"])

    def dict(self):
        d = super(Predicate, self).dict()
        d["text"] = self.text
        return d


class Random(Condition):
    predicate = "c_random_less"

    def __init__(self, percent=10):
        super(Random, self).__init__(Random.predicate)
        self.percent = percent
        self.params.append(self.percent)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["percent"])

    def dict(self):
        d = super(Random, self).dict()
        d["percent"] = self.percent
        return d


class SignalFlareInZone(Condition):
    predicate = "c_signal_flare_in_zone"

    def __init__(self, flare_color, numflares, zone):
        super(SignalFlareInZone, self).__init__(SignalFlareInZone.predicate)
        self.flare_color = flare_color
        self.params.append(self.flare_color)
        self.numflares = numflares
        self.params.append(self.numflares)
        self.zone = zone
        self.params.append(self.zone)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["flare_color"], d["numflares"], d["zone"])

    def dict(self):
        d = super(SignalFlareInZone, self).dict()
        d["flare_color"] = self.flare_color
        d["numflares"] = self.numflares
        d["zone"] = self.zone
        return d


class TimeAfter(Condition):
    predicate = "c_time_after"

    def __init__(self, seconds=10):
        super(TimeAfter, self).__init__(TimeAfter.predicate)
        self.seconds = seconds
        self.params.append(self.seconds)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["seconds"])

    def dict(self):
        d = super(TimeAfter, self).dict()
        d["seconds"] = self.seconds
        return d


class TimeBefore(Condition):
    predicate = "c_time_before"

    def __init__(self, seconds=10):
        super(TimeBefore, self).__init__(TimeBefore.predicate)
        self.seconds = seconds
        self.params.append(self.seconds)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["seconds"])

    def dict(self):
        d = super(TimeBefore, self).dict()
        d["seconds"] = self.seconds
        return d


class TimeSinceFlag(Condition):
    predicate = "c_time_since_flag"

    def __init__(self, flag=1, seconds=10):
        super(TimeSinceFlag, self).__init__(TimeSinceFlag.predicate)
        self.flag = flag
        self.params.append(self.flag)
        self.seconds = seconds
        self.params.append(self.seconds)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["flag"], d["seconds"])

    def dict(self):
        d = super(TimeSinceFlag, self).dict()
        d["flag"] = self.flag
        d["seconds"] = self.seconds
        return d


class UnitAlive(Condition):
    predicate = "c_unit_alive"

    def __init__(self, unit):
        super(UnitAlive, self).__init__(UnitAlive.predicate)
        self.unit = unit
        self.params.append(self.unit)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["unit"])

    def dict(self):
        d = super(UnitAlive, self).dict()
        d["unit"] = self.unit
        return d


class UnitAltitudeHigher(Condition):
    predicate = "c_unit_altitude_higher"

    def __init__(self, unit, altitude=1):
        super(UnitAltitudeHigher, self).__init__(UnitAltitudeHigher.predicate)
        self.unit = unit
        self.params.append(self.unit)
        self.altitude = altitude
        self.params.append(self.altitude)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["unit"], d["altitude"])

    def dict(self):
        d = super(UnitAltitudeHigher, self).dict()
        d["unit"] = self.unit
        d["altitude"] = self.altitude
        return d


class UnitAltitudeLower(Condition):
    predicate = "c_unit_altitude_lower"

    def __init__(self, unit, altitude=1):
        super(UnitAltitudeLower, self).__init__(UnitAltitudeLower.predicate)
        self.unit = unit
        self.params.append(self.unit)
        self.altitude = altitude
        self.params.append(self.altitude)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["unit"], d["altitude"])

    def dict(self):
        d = super(UnitAltitudeLower, self).dict()
        d["unit"] = self.unit
        d["altitude"] = self.altitude
        return d


class UnitBankWithin(Condition):
    predicate = "c_unit_bank"

    def __init__(self, unit, min_unit_bank=-180, max_unit_bank=180):
        super(UnitBankWithin, self).__init__(UnitBankWithin.predicate)
        self.unit = unit
        self.params.append(self.unit)
        self.min_unit_bank = min_unit_bank
        self.params.append(self.min_unit_bank)
        self.max_unit_bank = max_unit_bank
        self.params.append(self.max_unit_bank)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["unit"], d["min_unit_bank"], d["max_unit_bank"])

    def dict(self):
        d = super(UnitBankWithin, self).dict()
        d["unit"] = self.unit
        d["min_unit_bank"] = self.min_unit_bank
        d["max_unit_bank"] = self.max_unit_bank
        return d


class UnitDamaged(Condition):
    predicate = "c_unit_damaged"

    def __init__(self, unit):
        super(UnitDamaged, self).__init__(UnitDamaged.predicate)
        self.unit = unit
        self.params.append(self.unit)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["unit"])

    def dict(self):
        d = super(UnitDamaged, self).dict()
        d["unit"] = self.unit
        return d


class UnitDead(Condition):
    predicate = "c_unit_dead"

    def __init__(self, unit):
        super(UnitDead, self).__init__(UnitDead.predicate)
        self.unit = unit
        self.params.append(self.unit)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["unit"])

    def dict(self):
        d = super(UnitDead, self).dict()
        d["unit"] = self.unit
        return d


class UnitHeadingWithin(Condition):
    predicate = "c_unit_heading"

    def __init__(self, unit, min_unit_heading, max_unit_heading=360):
        super(UnitHeadingWithin, self).__init__(UnitHeadingWithin.predicate)
        self.unit = unit
        self.params.append(self.unit)
        self.min_unit_heading = min_unit_heading
        self.params.append(self.min_unit_heading)
        self.max_unit_heading = max_unit_heading
        self.params.append(self.max_unit_heading)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["unit"], d["min_unit_heading"], d["max_unit_heading"])

    def dict(self):
        d = super(UnitHeadingWithin, self).dict()
        d["unit"] = self.unit
        d["min_unit_heading"] = self.min_unit_heading
        d["max_unit_heading"] = self.max_unit_heading
        return d


class UnitInMovingZone(Condition):
    predicate = "c_unit_in_zone_unit"

    def __init__(self, unit, zone, zoneunit):
        super(UnitInMovingZone, self).__init__(UnitInMovingZone.predicate)
        self.unit = unit
        self.params.append(self.unit)
        self.zone = zone
        self.params.append(self.zone)
        self.zoneunit = zoneunit
        self.params.append(self.zoneunit)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["unit"], d["zone"], d["zoneunit"])

    def dict(self):
        d = super(UnitInMovingZone, self).dict()
        d["unit"] = self.unit
        d["zone"] = self.zone
        d["zoneunit"] = self.zoneunit
        return d


class UnitInZone(Condition):
    predicate = "c_unit_in_zone"

    def __init__(self, unit, zone):
        super(UnitInZone, self).__init__(UnitInZone.predicate)
        self.unit = unit
        self.params.append(self.unit)
        self.zone = zone
        self.params.append(self.zone)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["unit"], d["zone"])

    def dict(self):
        d = super(UnitInZone, self).dict()
        d["unit"] = self.unit
        d["zone"] = self.zone
        return d


class UnitLifeLess(Condition):
    predicate = "c_unit_life_less"

    def __init__(self, unit, percent=10):
        super(UnitLifeLess, self).__init__(UnitLifeLess.predicate)
        self.unit = unit
        self.params.append(self.unit)
        self.percent = percent
        self.params.append(self.percent)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["unit"], d["percent"])

    def dict(self):
        d = super(UnitLifeLess, self).dict()
        d["unit"] = self.unit
        d["percent"] = self.percent
        return d


class UnitOutsideMovingZone(Condition):
    predicate = "c_unit_out_zone_unit"

    def __init__(self, unit, zone, zoneunit):
        super(UnitOutsideMovingZone, self).__init__(UnitOutsideMovingZone.predicate)
        self.unit = unit
        self.params.append(self.unit)
        self.zone = zone
        self.params.append(self.zone)
        self.zoneunit = zoneunit
        self.params.append(self.zoneunit)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["unit"], d["zone"], d["zoneunit"])

    def dict(self):
        d = super(UnitOutsideMovingZone, self).dict()
        d["unit"] = self.unit
        d["zone"] = self.zone
        d["zoneunit"] = self.zoneunit
        return d


class UnitOutsideZone(Condition):
    predicate = "c_unit_out_zone"

    def __init__(self, unit, zone):
        super(UnitOutsideZone, self).__init__(UnitOutsideZone.predicate)
        self.unit = unit
        self.params.append(self.unit)
        self.zone = zone
        self.params.append(self.zone)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["unit"], d["zone"])

    def dict(self):
        d = super(UnitOutsideZone, self).dict()
        d["unit"] = self.unit
        d["zone"] = self.zone
        return d


class UnitPitchWithin(Condition):
    predicate = "c_unit_pitch"

    def __init__(self, unit, min_unit_pitch=-180, max_unit_pitch=180):
        super(UnitPitchWithin, self).__init__(UnitPitchWithin.predicate)
        self.unit = unit
        self.params.append(self.unit)
        self.min_unit_pitch = min_unit_pitch
        self.params.append(self.min_unit_pitch)
        self.max_unit_pitch = max_unit_pitch
        self.params.append(self.max_unit_pitch)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["unit"], d["min_unit_pitch"], d["max_unit_pitch"])

    def dict(self):
        d = super(UnitPitchWithin, self).dict()
        d["unit"] = self.unit
        d["min_unit_pitch"] = self.min_unit_pitch
        d["max_unit_pitch"] = self.max_unit_pitch
        return d


class UnitSpeedHigher(Condition):
    predicate = "c_unit_speed_higher"

    def __init__(self, unit, speed=100):
        super(UnitSpeedHigher, self).__init__(UnitSpeedHigher.predicate)
        self.unit = unit
        self.params.append(self.unit)
        self.speed = speed
        self.params.append(self.speed)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["unit"], d["speed"])

    def dict(self):
        d = super(UnitSpeedHigher, self).dict()
        d["unit"] = self.unit
        d["speed"] = self.speed
        return d


class UnitSpeedLower(Condition):
    predicate = "c_unit_speed_lower"

    def __init__(self, unit, speed=100):
        super(UnitSpeedLower, self).__init__(UnitSpeedLower.predicate)
        self.unit = unit
        self.params.append(self.unit)
        self.speed = speed
        self.params.append(self.speed)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["unit"], d["speed"])

    def dict(self):
        d = super(UnitSpeedLower, self).dict()
        d["unit"] = self.unit
        d["speed"] = self.speed
        return d


class UnitVerticalSpeedWithin(Condition):
    predicate = "c_unit_vertical_speed"

    def __init__(self, unit, min_unit_vertical_speed=-300, max_unit_vertical_speed=300):
        super(UnitVerticalSpeedWithin, self).__init__(UnitVerticalSpeedWithin.predicate)
        self.unit = unit
        self.params.append(self.unit)
        self.min_unit_vertical_speed = min_unit_vertical_speed
        self.params.append(self.min_unit_vertical_speed)
        self.max_unit_vertical_speed = max_unit_vertical_speed
        self.params.append(self.max_unit_vertical_speed)

    @classmethod
    def create_from_dict(cls, d):
        return cls(d["unit"], d["min_unit_vertical_speed"], d["max_unit_vertical_speed"])

    def dict(self):
        d = super(UnitVerticalSpeedWithin, self).dict()
        d["unit"] = self.unit
        d["min_unit_vertical_speed"] = self.min_unit_vertical_speed
        d["max_unit_vertical_speed"] = self.max_unit_vertical_speed
        return d


condition_map = {
    "c_all_of_coalition_in_zone": AllOfCoalitionInZone,
    "c_all_of_coalition_out_zone": AllOfCoalitionOutsideZone,
    "c_all_of_group_in_zone": AllOfGroupInZone,
    "c_all_of_group_out_zone": AllOfGroupOutsideZone,
    "c_argument_in_range": ArgumentInRange,
    "c_bomb_in_zone": BombInZone,
    "c_cargo_unhooked_in_zone": CargoUnhookedInZone,
    "c_coalition_has_airdrome": CoalitionHasAirdrome,
    "c_coalition_has_helipad": CoalitionHasHelipad,
    "c_cockpit_highlight_visible": CockpitHighlightVisible,
    "c_cockpit_param_equal_to": CockpitParamEqual,
    "c_cockpit_param_is_equal_to_another": CockpitParamEqualToAnother,
    "c_cockpit_param_in_range": CockpitParamInRange,
    "c_flag_equals": FlagEquals,
    "c_flag_equals_flag": FlagEqualsFlag,
    "c_flag_is_false": FlagIsFalse,
    "c_flag_less": FlagIsLess,
    "c_flag_less_flag": FlagIsLessThanFlag,
    "c_flag_more": FlagIsMore,
    "c_flag_is_true": FlagIsTrue,
    "c_group_alive": GroupAlive,
    "c_group_dead": GroupDead,
    "c_group_life_less": GroupLifeLess,
    "c_indication_txt_equal_to": IndicationTextEqual,
    "c_mission_score_higher": MissionScoreHigher,
    "c_mission_score_lower": MissionScoreLower,
    "or": Or,
    "c_part_of_coalition_in_zone": PartOfCoalitionInZone,
    "c_part_of_coalition_out_zone": PartOfCoalitionOutsideZone,
    "c_part_of_group_in_zone": PartOfGroupInZone,
    "c_part_of_group_out_zone": PartOfGroupOutsideZone,
    "c_player_score_less": PlayerScoreLess,
    "c_player_score_more": PlayerScoreMore,
    "c_predicate": Predicate,
    "c_random_less": Random,
    "c_signal_flare_in_zone": SignalFlareInZone,
    "c_time_after": TimeAfter,
    "c_time_before": TimeBefore,
    "c_time_since_flag": TimeSinceFlag,
    "c_unit_alive": UnitAlive,
    "c_unit_altitude_higher": UnitAltitudeHigher,
    "c_unit_altitude_lower": UnitAltitudeLower,
    "c_unit_bank": UnitBankWithin,
    "c_unit_damaged": UnitDamaged,
    "c_unit_dead": UnitDead,
    "c_unit_heading": UnitHeadingWithin,
    "c_unit_in_zone_unit": UnitInMovingZone,
    "c_unit_in_zone": UnitInZone,
    "c_unit_life_less": UnitLifeLess,
    "c_unit_out_zone_unit": UnitOutsideMovingZone,
    "c_unit_out_zone": UnitOutsideZone,
    "c_unit_pitch": UnitPitchWithin,
    "c_unit_speed_higher": UnitSpeedHigher,
    "c_unit_speed_lower": UnitSpeedLower,
    "c_unit_vertical_speed": UnitVerticalSpeedWithin,
}



from typing import List
from . import condition


class Goal:
    def __init__(self, comment="", score=100):
        self.rules = []  # type: List[condition.Condition]
        self.side = "OFFLINE"
        self.score = score
        self.predicate = "score"
        self.comment = comment

    def load_from_dict(self, data):
        self.side = data["side"]
        self.score = data["score"]
        self.predicate = data["predicate"]
        self.comment = data["comment"]
        self.rules = []
        rules = data["rules"]
        for x in rules:
            gr = condition.condition_map[rules[x]["predicate"]].create_from_dict(rules[x])
            self.rules.append(gr)

    def conditions(self):
        return " and ".join(map(repr, self.rules))

    def dict(self):
        return {
            "side": self.side,
            "score": self.score,
            "predicate": self.predicate,
            "comment": self.comment,
            "rules": {i+1: self.rules[i].dict() for i in range(0, len(self.rules))}
        }


class Goals:
    def __init__(self):
        self.goals = {
            "red": [],  # type list[Goal]
            "blue": [],  # type list[Goal]
            "offline": []  # type list[Goal]
        }

    def load_from_dict(self, data):
        for x in data:
            g = Goal()
            g.load_from_dict(data[x])
            self.goals[data[x]["side"].lower()].append(g)

    def add_red(self, g: Goal):
        g.side = "RED"
        self.goals["red"].append(g)

    def add_blue(self, g: Goal):
        g.side = "BLUE"
        self.goals["blue"].append(g)

    def add_offline(self, g: Goal):
        g.side = "OFFLINE"
        self.goals["offline"].append(g)

    def generate_result(self):
        d = {
            "total": len(self.goals["blue"]) + len(self.goals["red"]) + len(self.goals["offline"]),
            "blue": {},
            "red": {},
            "offline": {}
        }
        funcstr = "if mission.result.{side}.conditions[{idx}]() then mission.result.{side}.actions[{idx}]() end"
        for side in ["blue", "red", "offline"]:
            d[side]["conditions"] = {i + 1: "return(" + self.goals[side][i].conditions() + ")"
                                     for i in range(0, len(self.goals[side]))}
            d[side]["actions"] = {i + 1: "a_set_mission_result(" + str(self.goals[side][i].score) + ")"
                                  for i in range(0, len(self.goals[side])) if self.goals[side][i].rules}
            d[side]["func"] = {
                i + 1: funcstr.format(side=side, idx=i+1)
                for i in range(0, len(self.goals[side])) if self.goals[side][i].rules}
        return d

    def dict(self):
        return {i+1: self.goals[side][i].dict() for side in ["blue", "red", "offline"] for i in range(0, len(self.goals[side]))}
